@@ -12,7 +12,7 @@ template.initializer = 'initialize'
 )]
 pub struct Asset(&'static str, &'static str);
 
-pub fn main() {
+fn main() {
     use base::examples::assets;
 
     assert_eq!(assets::_ENV.0, "examples/assets/.env");
