@@ -8,7 +8,7 @@ pub struct Asset {
     relative_path: &'static str,
 }
 
-pub fn main() {
+fn main() {
     assert_eq!(
         base::levels::TUTORIAL_JSON.relative_path,
         "levels/tutorial.json",
