@@ -37,7 +37,7 @@ pub struct Asset {
     version: &'static str,
 }
 
-pub fn main() {
+fn main() {
     use base::examples::assets;
 
     assert_eq!(assets::CREDITS_MD.path_length, 26);
