@@ -10,8 +10,9 @@ macro_rules! visit_base {
 macro_rules! visit_folder {
     ($name:literal, $id:ident, $($contents:expr)*) => {{
         fn list_folder(depth: usize) -> String {
+            let name = $name;
             vec![
-                indent_line(depth, &format!("{}/", $name)),
+                indent_line(depth, &format!("{name}/")),
                 $($contents(depth + 1),)*
             ]
             .join("")
@@ -21,7 +22,8 @@ macro_rules! visit_folder {
 }
 
 fn indent_line(depth: usize, contents: &str) -> String {
-    format!("{}{}\n", "    ".repeat(depth), contents)
+    let indentation = "    ".repeat(depth);
+    format!("{indentation}{contents}\n")
 }
 
 macro_rules! visit_file {
