@@ -6,8 +6,10 @@
 //! - File lookups **checked** at compile time are **fast** at runtime (constant
 //!   time).
 //! - **Customizable**: associate any data with files.
-//! - Many [**examples**](https://github.com/evolutics/iftree/tree/main/examples).
-//! - See also [**related projects**](#related-work).
+//! - Many [**examples**](https://github.com/evolutics/iftree/tree/main/examples),
+//!   including [recipes](#recipes).
+//!
+//! See also [**related projects**](#related-work).
 //!
 //! ![Test](https://github.com/evolutics/iftree/actions/workflows/test.yml/badge.svg)
 //! [![crates.io](https://img.shields.io/crates/v/iftree.svg)](https://crates.io/crates/iftree)
@@ -34,11 +36,14 @@
 //! Iftree:    any_macro!("my_files/**")
 //! ```
 //!
-//! # Introduction
+//! # Usage
 //!
-//! Here is a minimal example that shows the basic functionality.
+//! Now that you know the why and the what, learn the how. The following quick start
+//! shows the basic usage.
 //!
-//! ```ignore
+//! ## Quick start
+//!
+//! ```rust
 //! // Say you have these files:
 //! //
 //! //     my_assets/
@@ -47,36 +52,30 @@
 //! //     └── folder/
 //! //         └── file_c
 //!
-//! // To include this file tree in your code, define a custom type like so:
+//! // Include data from this file tree in your code like so:
 //! #[iftree::include_file_tree("paths = '/my_assets/**'")]
 //! pub struct MyAsset {
+//!     relative_path: &'static str,
 //!     contents_str: &'static str,
 //! }
-//! // Above we configure a path pattern to filter the files in `my_assets/` and its
-//! // subfolders. For each selected file, an instance of `MyAsset` is initialized.
-//! // The standard field `contents_str` is automatically populated with a call to
-//! // `include_str!`, but you can plug in your own initializer.
 //!
 //! fn main() {
-//!     // Based on this, an array `ASSETS` with the desired file data is generated:
+//!     // Based on this, an array `ASSETS` of `MyAsset` instances is generated:
 //!     assert_eq!(ASSETS.len(), 3);
+//!     assert_eq!(ASSETS[0].relative_path, "my_assets/file_a");
 //!     assert_eq!(ASSETS[0].contents_str, "… contents file_a\n");
 //!     assert_eq!(ASSETS[1].contents_str, "… contents file_b\n");
 //!     assert_eq!(ASSETS[2].contents_str, "… file_c\n");
 //!
 //!     // Also, variables `base::x::y::MY_FILE` are generated (named by file path):
+//!     assert_eq!(base::my_assets::FILE_A.relative_path, "my_assets/file_a");
 //!     assert_eq!(base::my_assets::FILE_A.contents_str, "… contents file_a\n");
 //!     assert_eq!(base::my_assets::FILE_B.contents_str, "… contents file_b\n");
 //!     assert_eq!(base::my_assets::folder::FILE_C.contents_str, "… file_c\n");
 //! }
 //! ```
 //!
-//! # Usage
-//!
-//! Now that you have a general idea of the library, learn how to integrate it with
-//! your project.
-//!
-//! ## Getting started
+//! ## Detailed guide
 //!
 //! 1. Add the **dependency** `iftree = "1.0"` to your manifest (`Cargo.toml`).
 //!
@@ -84,14 +83,14 @@
 //!    Example:
 //!
 //!    ```rust
-//!    pub struct Asset;
+//!    pub struct MyAsset;
 //!    ```
 //!
 //! 1. Next, **filter files** to be included by annotating your asset type. Example:
 //!
 //!    ```ignore
 //!    #[iftree::include_file_tree("paths = '/my_assets/**'")]
-//!    pub struct Asset;
+//!    pub struct MyAsset;
 //!    ```
 //!
 //!    The macro argument is a [TOML](https://toml.io) string literal. Its `paths`
@@ -103,29 +102,37 @@
 //!
 //!    ```ignore
 //!    #[iftree::include_file_tree("paths = '/my_assets/**'")]
-//!    pub struct Asset {
+//!    pub struct MyAsset {
 //!        relative_path: &'static str,
 //!        contents_bytes: &'static [u8],
 //!    }
 //!    ```
 //!
-//!    When building your project, code is generated that uses an initializer to
-//!    instantiate the asset type once per file.
+//!    When building your project, code is generated that instantiates the asset
+//!    type once per file.
 //!
-//!    By default, a field `contents_bytes` (if any) is populated with the file
-//!    contents as a byte array, a field `contents_str` is populated with the file
-//!    contents as a string, and a couple of other
-//!    [standard fields](#standard-fields) are recognized.
+//!    By default, a field `relative_path` (if any) is populated with the file path,
+//!    a field `contents_bytes` is populated with the raw file contents, and a
+//!    couple of other [standard fields](#standard-fields) are recognized by name.
 //!
-//!    However, you can fully customize the initialization by
-//!    [configuring an initializer](#templateinitializer). For even more control
-//!    over code generation, there is the concept of [visitors](#template-visitors).
+//!    However, you can [customize](#custom-file-data) this to include arbitrary
+//!    file data.
 //!
-//! 1. Now you can **access** your included file data via `ASSETS` array or via
-//!    `base::x::y::MY_FILE` variables. Example:
+//! 1. Now you can **access** your file data via the generated `ASSETS` array.
+//!    Example:
+//!
+//!    ```ignore
+//!    assert_eq!(ASSETS[0].relative_path, "my_assets/my_file");
+//!    assert_eq!(ASSETS[0].contents_bytes, b"file contents");
+//!    ```
+//!
+//!    Additionally, for each file `x/y/my_file`, a variable `base::x::y::MY_FILE`
+//!    is generated (unless disabled via
+//!    [`template.identifiers` configuration](#templateidentifiers)). Such a
+//!    variable is a reference to the respective element of the `ASSETS` array.
+//!    Example:
 //!
 //!    ```ignore
-//!    println!("Number of assets: {}", ASSETS.len());
 //!    assert_eq!(base::my_assets::MY_FILE.relative_path, "my_assets/my_file");
 //!    assert_eq!(base::my_assets::MY_FILE.contents_bytes, b"file contents");
 //!    ```
@@ -137,7 +144,7 @@
 //! The documentation links to individual examples where helpful.
 //!
 //! You could get started with the
-//! [introductory example](https://github.com/evolutics/iftree/blob/main/examples/basic.rs).
+//! [basic example](https://github.com/evolutics/iftree/blob/main/examples/basic.rs).
 //! For a more complex case, see the
 //! [showcase example](https://github.com/evolutics/iftree/blob/main/examples/showcase.rs).
 //!
@@ -147,20 +154,20 @@
 //! ## Standard fields
 //!
 //! When you use a subset of the following fields only, an initializer for your
-//! asset type is generated without further configuration. You can still override
-//! these field names with a [custom initializer](#templateinitializer).
+//! asset type is generated without further configuration. See
+//! [example](https://github.com/evolutics/iftree/blob/main/examples/basics_standard_fields.rs).
 //!
-//! - **`contents_bytes`**: `&'static [u8]`
+//! - **`contents_bytes`**`: &'static [u8]`
 //!
 //!   File contents as a byte array, using
 //!   [`std::include_bytes`](https://doc.rust-lang.org/std/macro.include_bytes.html).
 //!
-//! - **`contents_str`**: `&'static str`
+//! - **`contents_str`**`: &'static str`
 //!
 //!   File contents interpreted as a UTF-8 string, using
 //!   [`std::include_str`](https://doc.rust-lang.org/std/macro.include_str.html).
 //!
-//! - **`get_bytes`**: `fn() -> std::borrow::Cow<'static, [u8]>`
+//! - **`get_bytes`**`: fn() -> std::borrow::Cow<'static, [u8]>`
 //!
 //!   In debug builds (that is, when
 //!   [`debug_assertions`](https://doc.rust-lang.org/reference/conditional-compilation.html#debug_assertions)
@@ -174,20 +181,57 @@
 //!   using
 //!   [`std::include_bytes`](https://doc.rust-lang.org/std/macro.include_bytes.html).
 //!
-//! - **`get_str`**: `fn() -> std::borrow::Cow<'static, str>`
+//! - **`get_str`**`: fn() -> std::borrow::Cow<'static, str>`
 //!
 //!   Same as `get_bytes` but for the file contents interpreted as a UTF-8 string,
 //!   using
 //!   [`std::include_str`](https://doc.rust-lang.org/std/macro.include_str.html).
 //!
-//! - **`relative_path`**: `&'static str`
+//! - **`relative_path`**`: &'static str`
 //!
 //!   File path relative to the base folder, which is the folder with your manifest
 //!   (`Cargo.toml`) by default. Path components are separated by a slash `/`,
 //!   independent of your platform.
 //!
-//! See
-//! [example](https://github.com/evolutics/iftree/blob/main/examples/basics_standard_fields.rs).
+//! ## Custom file data
+//!
+//! To associate custom data with your files, you can plug in a macro that
+//! initializes each asset. Toy example:
+//!
+//! ```rust
+//! macro_rules! my_initialize {
+//!     ($relative_path:literal, $absolute_path:literal) => {
+//!         MyAsset {
+//!             path: $relative_path,
+//!             size_in_bytes: include_bytes!($absolute_path).len(),
+//!         }
+//!     };
+//! }
+//!
+//! #[iftree::include_file_tree(
+//!     "
+//! paths = '/my_assets/**'
+//! template.initializer = 'my_initialize'
+//! "
+//! )]
+//! pub struct MyAsset {
+//!     path: &'static str,
+//!     size_in_bytes: usize,
+//! }
+//!
+//! fn main() {
+//!     assert_eq!(base::my_assets::FILE_A.path, "my_assets/file_a");
+//!     assert_eq!(base::my_assets::FILE_A.size_in_bytes, 20);
+//!     assert_eq!(base::my_assets::FILE_B.path, "my_assets/file_b");
+//! }
+//! ```
+//!
+//! The initializer macro (`my_initialize` above) must return a constant expression.
+//! Non-constant data can still be computed (lazily) with a library like
+//! [`once_cell`](https://github.com/matklad/once_cell).
+//!
+//! For even more control over code generation, there is the concept of
+//! [visitors](#template-visitors).
 //!
 //! ## Name sanitization
 //!
@@ -195,7 +239,7 @@
 //! filename `404_not_found.md` is sanitized to an identifier `_404_NOT_FOUND_MD`.
 //!
 //! The sanitization process is designed to generate valid, conventional
-//! [Unicode identifiers](https://doc.rust-lang.org/reference/identifiers.html).
+//! [identifiers](https://doc.rust-lang.org/reference/identifiers.html).
 //! Essentially, it replaces invalid identifier characters by underscores `"_"` and
 //! adjusts the letter case to the context.
 //!
@@ -211,10 +255,6 @@
 //! 1. If the name is `"_"`, `"crate"`, `"self"`, `"Self"`, or `"super"`, then `"_"`
 //!    is appended.
 //!
-//! Note that non-ASCII identifiers are only supported from Rust 1.53.0. For earlier
-//! versions, the sanitization here may generate invalid identifiers if you use
-//! non-ASCII paths, in which case you need to manually rename any affected files.
-//!
 //! ## Portable file paths
 //!
 //! To prevent issues when developing on different platforms, your file paths should
@@ -244,20 +284,33 @@
 //!
 //! ## Integration with other libraries
 //!
-//! - Compression with [`include_flate`](https://github.com/evolutics/iftree/blob/main/examples/library_include_flate.rs)
-//! - File server with [Actix Web](https://github.com/evolutics/iftree/blob/main/examples/library_actix_web.rs)
-//! - File server with [Rocket](https://github.com/evolutics/iftree/blob/main/examples/library_rocket.rs)
-//! - File server with [Tide](https://github.com/evolutics/iftree/blob/main/examples/library_tide.rs)
-//! - File server with [warp](https://github.com/evolutics/iftree/blob/main/examples/library_warp.rs)
-//! - Lazy initialization with [`lazy_static`](https://github.com/evolutics/iftree/blob/main/examples/library_lazy_static.rs)
-//! - Lazy initialization with [`once_cell`](https://github.com/evolutics/iftree/blob/main/examples/library_once_cell.rs)
-//! - Media types with [`mime_guess`](https://github.com/evolutics/iftree/blob/main/examples/library_mime_guess.rs)
-//! - Templates with [Handlebars](https://github.com/evolutics/iftree/blob/main/examples/library_handlebars.rs)
+//! - Compression with
+//!   [`include_flate`](https://github.com/evolutics/iftree/blob/main/examples/library_include_flate.rs)
+//! - File server with
+//!   [Actix Web](https://github.com/evolutics/iftree/blob/main/examples/library_actix_web.rs)
+//! - File server with
+//!   [Rocket](https://github.com/evolutics/iftree/blob/main/examples/library_rocket.rs)
+//! - File server with
+//!   [Tide](https://github.com/evolutics/iftree/blob/main/examples/library_tide.rs)
+//! - File server with
+//!   [warp](https://github.com/evolutics/iftree/blob/main/examples/library_warp.rs)
+//! - Lazy initialization with
+//!   [`lazy_static`](https://github.com/evolutics/iftree/blob/main/examples/library_lazy_static.rs)
+//! - Lazy initialization with
+//!   [`once_cell`](https://github.com/evolutics/iftree/blob/main/examples/library_once_cell.rs)
+//! - Media types with
+//!   [`mime_guess`](https://github.com/evolutics/iftree/blob/main/examples/library_mime_guess.rs)
+//! - Templates with
+//!   [Handlebars](https://github.com/evolutics/iftree/blob/main/examples/library_handlebars.rs)
 //!
 //! ## Including file metadata
 //!
+//! - [File permissions](https://github.com/evolutics/iftree/blob/main/examples/scenario_file_permissions.rs)
+//! - [File timestamps](https://github.com/evolutics/iftree/blob/main/examples/scenario_file_timestamps.rs)
+//!   (creation, last access, last modification)
 //! - [Filename](https://github.com/evolutics/iftree/blob/main/examples/scenario_filename.rs)
 //! - [Filename extension](https://github.com/evolutics/iftree/blob/main/examples/scenario_filename_extension.rs)
+//! - Hash with [SHA-256](https://github.com/evolutics/iftree/blob/main/examples/scenario_hash_sha_256.rs)
 //! - [Media type](https://github.com/evolutics/iftree/blob/main/examples/scenario_media_type.rs)
 //!   (formerly MIME type)
 //!
@@ -273,20 +326,18 @@
 //! has since developed into something more flexible.
 //!
 //! Here is how I think Iftree compares to related projects for the given criteria.
+//! Generally, while Iftree has defaults to address common use cases, it comes with
+//! first-class support for arbitrary file data.
 //!
 //! | Project                                                                            | File selection                                      | Included file data       | Data access via                                                                                     |
 //! | ---------------------------------------------------------------------------------- | --------------------------------------------------- | ------------------------ | --------------------------------------------------------------------------------------------------- |
-//! | [**`include_dir`**](https://github.com/Michael-F-Bryan/include_dir) 0.6            | Single folder                                       | Path, contents           | File path, nested iterators, glob patterns                                                          |
+//! | [**`include_dir`**](https://github.com/Michael-F-Bryan/include_dir) 0.7            | Single folder                                       | Path, contents, metadata | File path, nested iterators, glob patterns                                                          |
 //! | [**`includedir`**](https://github.com/tilpner/includedir) 0.6                      | Multiple files, multiple folders                    | Path, contents           | File path, iterator                                                                                 |
-//! | [**Rust Embed**](https://github.com/pyros2097/rust-embed) 6.0                      | Single folder                                       | Path, contents, metadata | File path, iterator                                                                                 |
+//! | [**Rust Embed**](https://github.com/pyros2097/rust-embed) 6.3                      | Single folder, inclusion-exclusion path patterns    | Path, contents, metadata | File path, iterator                                                                                 |
 //! | [**`std::include_bytes`**](https://doc.rust-lang.org/std/macro.include_bytes.html) | Single file                                         | Contents                 | File path                                                                                           |
 //! | [**`std::include_str`**](https://doc.rust-lang.org/std/macro.include_str.html)     | Single file                                         | Contents                 | File path                                                                                           |
 //! | **Iftree**                                                                         | Multiple files by inclusion-exclusion path patterns | Path, contents, custom   | File path (via `base::x::y::MY_FILE` variables in constant time), iterator (`ASSETS` array), custom |
 //!
-//! Generally, while Iftree has defaults to address common use cases, it can be
-//! customized to support more specific use cases, too (see [recipes](#recipes) for
-//! examples).
-//!
 //! # Configuration reference
 //!
 //! The `iftree::include_file_tree` macro is configured via a
@@ -296,9 +347,12 @@
 //!
 //! Path patterns are interpreted as relative to this folder.
 //!
-//! If this path itself is relative, then it is joined to the folder given by the
-//! environment variable `CARGO_MANIFEST_DIR`. That is, a relative path `x/y/z` has
-//! a full path `[CARGO_MANIFEST_DIR]/[base_folder]/x/y/z`.
+//! Unless this path is absolute, it is interpreted as relative to the folder given
+//! by the environment variable `CARGO_MANIFEST_DIR`. That is, a path pattern
+//! `x/y/z` resolves to `[CARGO_MANIFEST_DIR]/[base_folder]/x/y/z`.
+//!
+//! See the [`root_folder_variable` configuration](#root_folder_variable) to
+//! customize this.
 //!
 //! **Default**: `""`
 //!
@@ -330,13 +384,15 @@
 //! difference: you must use `x/y/*` instead of `x/y/` to include files in a folder
 //! `x/y/`; to also include subfolders (recursively), use `x/y/**`.
 //!
-//! Exclude hidden files with `!.*` as a pattern. Another common pattern is of the
-//! form `*.xyz` to include files with filename extension `xyz` only.
-//!
 //! By default, path patterns are relative to the environment variable
 //! `CARGO_MANIFEST_DIR`, which is the folder with your manifest (`Cargo.toml`). See
 //! the [`base_folder` configuration](#base_folder) to customize this.
 //!
+//! Common patterns:
+//!
+//! - Exclude hidden files: `!.*`
+//! - Include files with filename extension `xyz` only: `*.xyz`
+//!
 //! This is a **required** option without default.
 //!
 //! See
@@ -344,8 +400,8 @@
 //!
 //! ## `root_folder_variable`
 //!
-//! The name of the environment variable to use as the root folder for the
-//! [`base_folder` configuration](#base_folder).
+//! An environment variable that is used to resolve a relative
+//! [`base_folder`](#base_folder) to an absolute path.
 //!
 //! The value of the environment variable should be an absolute path.
 //!
@@ -355,19 +411,19 @@
 //!
 //! Whether to generate an identifier per file.
 //!
-//! Given a file `x/y/my_file`, a `static` variable `base::x::y::MY_FILE` is
+//! Given a file `x/y/my_file`, a static variable `base::x::y::MY_FILE` is
 //! generated, nested in modules for folders. Their root module is `base`, which
 //! represents the base folder.
 //!
-//! Each variable is a reference to the corresponding element in the `ASSETS` array.
+//! Each variable is a reference to the corresponding element of the `ASSETS` array.
 //!
 //! Generated identifiers are subject to [name sanitization](#name-sanitization).
-//! Because of this, there may be collisions in the generated code, causing an error
+//! Because of this, two files may map to the same identifier, causing an error
 //! about a name being defined multiple times. The code generation does not try to
 //! resolve such collisions automatically, as this would likely cause confusion
-//! about which identifier refers to which file. Instead, you need to manually
-//! rename any affected paths (assuming you need the generated identifiers at all –
-//! otherwise, you can just disable this with `template.identifiers = false`).
+//! about which identifier refers to which file. Instead, you need to rename any
+//! affected paths (but if you have no use for the generated identifiers, you can
+//! just disable them with `template.identifiers = false`).
 //!
 //! **Default**: `true`
 //!
@@ -452,7 +508,7 @@
 //! # Further resources
 //!
 //! - [Changelog](https://github.com/evolutics/iftree/blob/main/CHANGELOG.md)
-//! - [crates.io](https://crates.io/crates/iftree)
+//! - [Latest revision of this documentation](https://github.com/evolutics/iftree/blob/main/README.md)
 
 mod generate_view;
 mod go;
@@ -472,7 +528,7 @@ pub fn include_file_tree(
     let type_ = syn::parse_macro_input!(item);
 
     match go::main(configuration, item2, type_) {
-        Err(error) => panic!("{}", error),
+        Err(error) => panic!("{error}"),
         Ok(code) => code.into(),
     }
 }
@@ -483,7 +539,8 @@ mod tests {
 
     #[test]
     fn readme_includes_manifest_description() {
-        let embedded_description = format!("\n\n{}\n\n", env!("CARGO_PKG_DESCRIPTION"));
+        let description = env!("CARGO_PKG_DESCRIPTION");
+        let embedded_description = format!("\n\n{description}\n\n");
 
         let actual = get_readme().contains(&embedded_description);
 
@@ -496,21 +553,20 @@ mod tests {
 
     #[test]
     fn readme_includes_basic_example() {
-        let example = format!("```rust\n{}```\n", include_str!("../examples/basic.rs"));
+        let example = include_str!("../examples/basic.rs");
+        let embedded_example = format!("```rust\n{example}```\n");
 
-        let actual = get_readme().contains(&example);
+        let actual = get_readme().contains(&embedded_example);
 
         assert!(actual);
     }
 
     #[test]
     fn readme_refers_to_current_manifest_version() {
-        let dependency = format!(
-            "`{} = \"{}.{}\"`",
-            env!("CARGO_PKG_NAME"),
-            env!("CARGO_PKG_VERSION_MAJOR"),
-            env!("CARGO_PKG_VERSION_MINOR"),
-        );
+        let name = env!("CARGO_PKG_NAME");
+        let version_major = env!("CARGO_PKG_VERSION_MAJOR");
+        let version_minor = env!("CARGO_PKG_VERSION_MINOR");
+        let dependency = format!("`{name} = \"{version_major}.{version_minor}\"`");
 
         let actual = get_readme().contains(&dependency);
 
@@ -539,7 +595,8 @@ mod tests {
 
     #[test]
     fn changelog_contains_current_manifest_version() {
-        let version_section = format!("\n\n## {} – ", env!("CARGO_PKG_VERSION"));
+        let version = env!("CARGO_PKG_VERSION");
+        let version_section = format!("\n\n## {version} – ");
 
         let actual = get_changelog().contains(&version_section);
 
@@ -553,19 +610,19 @@ mod tests {
     #[test]
     fn module_documentation_corresponds_to_readme() {
         let mut actual = String::from("# Iftree: Include File Tree 🌳\n\n");
-        let mut is_empty = true;
+        let mut is_code_block = false;
         for line in include_str!("lib.rs").lines() {
             match line.strip_prefix("//!") {
                 None => break,
                 Some(line) => {
                     let line = line.strip_prefix(' ').unwrap_or(line);
-                    let line = if is_empty && line.starts_with('#') {
-                        format!("#{}", line)
+                    let line = if !is_code_block && line.starts_with('#') {
+                        format!("#{line}")
                     } else {
                         line.replace("```ignore", "```rust")
                     };
-                    actual.push_str(&format!("{}\n", line));
-                    is_empty = line.is_empty();
+                    actual.push_str(&format!("{line}\n"));
+                    is_code_block ^= line.starts_with("```");
                 }
             }
         }
