@@ -5,7 +5,7 @@ use std::path;
 
 impl PartialEq for main::IgnoreError {
     fn eq(&self, other: &Self) -> bool {
-        format!("{:?}", self) == format!("{:?}", other)
+        format!("{self:?}") == format!("{other:?}")
     }
 }
 
@@ -14,66 +14,56 @@ impl fmt::Display for main::Error {
         match self {
             main::Error::EnvironmentVariable { name, source } => write!(
                 formatter,
-                "Unable to get environment variable {:?}: {}",
-                name, source,
+                "Unable to get environment variable {name:?}: {source}",
             ),
 
-            main::Error::Ignore(main::IgnoreError(error)) => write!(formatter, "{}", error),
+            main::Error::Ignore(main::IgnoreError(error)) => write!(formatter, "{error}"),
 
-            main::Error::NoInitializer => {
-                write!(
-                    formatter,
-                    "No initializer. \
-                    Configure one with \"template.initializer = 'a_macro'\" or \
-                    use standard fields to generate a default initializer.",
-                )
-            }
+            main::Error::NoInitializer => formatter.write_str(
+                "No initializer. \
+                Configure one with \"template.initializer = 'a_macro'\" or \
+                use standard fields to generate a default initializer.",
+            ),
 
             main::Error::NonstandardField {
                 field,
                 standard_fields,
             } => {
+                let field = field.to_string();
+                let standard_fields = standard_fields
+                    .iter()
+                    .map(|field| {
+                        let field = field.to_string();
+                        format!("{field:?}")
+                    })
+                    .collect::<Vec<_>>()
+                    .join(", ");
                 write!(
                     formatter,
                     "Default initializer cannot be generated \
-                    as field {:?} is not standard. \
+                    as field {field:?} is not standard. \
                     Configure an initializer with \"template.initializer = 'a_macro'\" or \
-                    use standard fields only ({}).",
-                    field.to_string(),
-                    standard_fields
-                        .iter()
-                        .map(|field| format!("{:?}", field.to_string()))
-                        .collect::<Vec<_>>()
-                        .join(", "),
+                    use standard fields only ({standard_fields}).",
                 )
             }
 
-            main::Error::PathInvalidUnicode(path) => {
-                write!(
-                    formatter,
-                    "Path is not valid Unicode, consider renaming it: {:?}",
-                    path,
-                )
-            }
+            main::Error::PathInvalidUnicode(path) => write!(
+                formatter,
+                "Path is not valid Unicode, consider renaming it: {path:?}",
+            ),
 
-            main::Error::PathStripPrefix(error) => write!(formatter, "{}", error),
+            main::Error::PathStripPrefix(error) => write!(formatter, "{error}"),
 
-            main::Error::UnexpectedEmptyRelativePath { absolute_path } => {
-                write!(
-                    formatter,
-                    "Unexpected empty relative path for absolute path \
-                    (consider reporting this): {:?}",
-                    absolute_path,
-                )
-            }
+            main::Error::UnexpectedEmptyRelativePath { absolute_path } => write!(
+                formatter,
+                "Unexpected empty relative path for absolute path \
+                (consider reporting this): {absolute_path:?}",
+            ),
 
-            main::Error::UnexpectedPathCollision(path) => {
-                write!(
-                    formatter,
-                    "Unexpected path collision (consider reporting this): {:?}",
-                    path,
-                )
-            }
+            main::Error::UnexpectedPathCollision(path) => write!(
+                formatter,
+                "Unexpected path collision (consider reporting this): {path:?}",
+            ),
         }
     }
 }
